@@ -5,11 +5,14 @@
 use strict;
 package String::Slice;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 use Exporter 'import';
 our @EXPORT = qw(slice);
 
+use Config;
+use Inline C => Config => ccflags => $Config::Config{ccflags} . " -Wall";
+
 use String::Slice::Inline C => <<'...';
 int slice (SV* dummy, ...) {
   dVAR; dXSARGS;
@@ -30,7 +33,6 @@ int slice (SV* dummy, ...) {
 
     // Set up local variables:
     U8* slice_ptr = SvPVX(slice);
-    U8* slice_end;
     I32 slice_off;
 
     U8* string_ptr = SvPVX(string);
@@ -48,6 +50,9 @@ int slice (SV* dummy, ...) {
       // Link the refcnt of string to slice:  (rafl++)
       sv_magicext(slice, string, PERL_MAGIC_ext, NULL, NULL, 0);
 
+      // Special way to tell perl it doesn't own the slice memory:  (jdb++)
+      SvLEN_set(slice, 0);
+
       // Make slice be utf8 if string is utf8:
       if (SvUTF8(string))
         SvUTF8_on(slice);
@@ -65,8 +70,12 @@ int slice (SV* dummy, ...) {
       slice_off = SvIVX(slice);
     }
 
-    // Hop to the new offset:
-    slice_ptr = utf8_hop(base_ptr, (offset - slice_off));
+    if (SvUTF8(string)) {
+      // Hop to the new offset:
+      slice_ptr = utf8_hop(base_ptr, (offset - slice_off));
+    } else {
+      slice_ptr = base_ptr + (offset - slice_off);
+    }
 
     // New offset is out of bounds. Handle failure:
     if (slice_ptr < string_ptr || slice_ptr > string_end) {
@@ -86,21 +95,23 @@ int slice (SV* dummy, ...) {
       // Set the slice character offset (sneaky hack into IV slot):
       SvIVX(slice) = offset;
 
-      // Calculate the proper byte length for the utf8 slice:
+      // Calculate and set the proper byte length for the utf8 slice:
 
-      // If requested number of chars is negative (default) or too big,
-      // use the entire remainder of the string:
-      if (length < 0 || length >= utf8_distance(string_end, slice_ptr))
-        slice_end = string_end;
-      // Else find the end of utf8 slice:
-      else
-        slice_end = utf8_hop(slice_ptr, length);
-
-      // Set the length of the slice buffer in bytes:
-      SvCUR_set(slice, slice_end - slice_ptr);
-
-      // Special way to tell perl it doesn't own the slice memory:  (jdb++)
-      SvLEN_set(slice, 0);
+      if (length < 0) {
+          SvCUR_set(slice, string_end - slice_ptr);
+      }
+      else if (SvUTF8(string)) {
+        if (length >= utf8_distance(string_end, slice_ptr)) {
+          SvCUR_set(slice, string_end - slice_ptr);
+        }
+        else
+          SvCUR_set(slice, utf8_hop(slice_ptr, length) - slice_ptr);
+      } else {
+        if (length >= string_end - slice_ptr)
+          SvCUR_set(slice, string_end - slice_ptr);
+        else
+          SvCUR_set(slice, length);
+      }
 
       // Success:
       return 1;
